.setcpu "65816"
.smart

; See MetaspriteTest2.asm
.importzp Joy1Data, Joy2Data, Joy1Press, Joy2Press

; Used by MetaspriteTest2.asm
.export JoyInit, GetInput

;------------------------------------------------------------------------
;-  Written by: Neviksti
;-     If you use my code, please share your creations with me
;-     as I am always curious :)
;------------------------------------------------------------------------

HVBJOY  = $4212
JOY0    = $4218
JOY1    = $421a
JOY2    = $421c
JOY3    = $421e
JOYSER0 = $4016
JOYSER1 = $4017
WRIO    = $4201

.segment "CODE"

.proc JoyInit
	php	

	rep	#$10	;8 bit mem/A, 16 bit X/Y
	sep	#$20

	lda #$c0	;have the automatic read of the SNES read the first pair of JoyPads
	sta WRIO

	ldx #$0000
	stx Joy1Press
	stx Joy2Press

;******* in ZSNES, the auto-read doesn't work unless NMI is enabled... is this correct?
;
;	lda $4212
;	lda $4212
;
;_W11:
;	lda $4212
;	and #$01
;	beq _W11		;Wait till automatic JoyPort read starts
;
;	jsr GetInput	;fill the variables with real Joypad data
;
;_W12:
;	lda $4212
;	and #$01
;	beq _W12		;Wait till automatic JoyPort read starts
;
;	jsr GetInput	;fill the variables with real Joypad data

	plp
	rts
.endproc


.proc GetInput
	php
	sep #$20   ; A=8

	lda #$01
:   bit $4212
	bne :-		;Wait till automatic JoyPort read is complete

	rep #$30   ; A=16, X/Y=16

	; ********** Get Joypads 1,2

	lda JOY0		;get JoyPad1
	tax 			
	eor Joy1Data		;A = A xor JoyState = (changes in joy state)
	stx Joy1Data		;update JoyState
	ora Joy1Press	;A = (joy changes) or (buttons pressed)
	and Joy1Data		;A = ((joy changes) or (buttons pressed)) and (current joy state)  
	sta Joy1Press	;store A = (buttons pressed since last clearing reg) and (button is still down)

	lda JOY1		;get JoyPad2
	tax 			
	eor Joy2Data		;A = A xor JoyState = (changes in joy state)
	stx Joy2Data		;update JoyState
	ora Joy2Press	;A = (joy changes) or (buttons pressed)
	and Joy2Data		;A = ((joy changes) or (buttons pressed)) and (current joy state)  
	sta Joy2Press	;store A = (buttons pressed since last clearing reg) and (button is still down)

	; ********** Make sure Joypads 1,2 are valid

	sep #$30   ; A=8, X/Y=8
	lda JOYSER0
	eor #$01
	and #$01		; A = -bit0 of JoySer0
	ora Joy1Data		
	sta Joy1Data		; joy state = (joy state) or A.... so bit0 of Joy1State = 0 only if it is a valid joypad

	lda JOYSER1
	eor #$01
	and #$01		; A = -bit0 of JoySer1
	ora Joy2Data		
	sta Joy2Data		; joy state = (joy state) or A.... so bit0 of Joy1State = 0 only if it is a valid joypad

	; ********** Change all invalid joypads to have a state of no button presses

	rep #$30   ; A=16, X/Y=16

	ldx #$0001
	lda #$000f

	bit Joy1Data		; A = joy state, if any of the bottom 4 bits are on... either nothing is plugged
	beq :+				; into the joy port, or it is not a joypad
	stx Joy1Data		; if it is not a valid joypad put $0001 into the 2 joy state variables
	stz Joy1Press

:	bit Joy2Data		; A = joy state, if any of the bottom 4 bits are on... either nothing is plugged
	beq _done			; into the joy port, or it is not a joypad
	stx Joy2Data		; if it is not a valid joypad put $0001 into the 2 joy state variables
	stz Joy2Press

_done:
	plp
	rts
.endproc
