.setcpu "65816"
.smart
.feature force_range

.include "Macros.asm"

; See 2input.asm
.import JoyInit, GetInput

; See Metasprite.asm
.import start_metasprite

; See ProcessOam.asm
.import start_process_oam

; See Objects.asm
.import start_object_spawner
.import start_object_identifier

; Used by 2input.asm
.exportzp Joy1Data, Joy2Data, Joy1Press, Joy2Press

; Used by Metasprite.asm
.exportzp Empty
.exportzp MetaspriteTableOffset
.exportzp MetaspriteCharacterOffset
.exportzp MetaspriteCount
.exportzp HFlipMask
.exportzp VFlipMask
.exportzp SpriteCount
.export SpriteBuf1, SpriteBuf2, SpriteBuf3, MetaspriteTable, MetaspriteTableSize, Bullet1MetaspriteTable, Bullet1MetaspriteTableSize

; Used by Objects.asm
.exportzp Joy1Data
.exportzp MapX
.exportzp MapY
.exportzp NewObjectRequest
.exportzp ObjectOffset
.export ObjectTable

; Used by Header.asm
.export VBlank, Main

;
; Game variables, located within 6502 zero page ($00-ff)
; 65816 has direct page which can range from $0000-ffff in region, but ca65
; only seems to understand "zp" being up to 256 bytes in size.
;
; See lorom256k.cfg and lda/tcd code in _reset
;
.segment "ZEROPAGE"

Empty:                     .res 2
FrameNum:                  .res 2
TempX:                     .res 2
TempY:                     .res 2
Joy1Data:                  .res 2
Joy2Data:                  .res 2
Joy1Press:                 .res 2
Joy2Press:                 .res 2
MapX:                      .res 2
MapY:                      .res 2
MetaspriteTableOffset:     .res 2
MetaspriteCharacterOffset: .res 2
MetaspriteCount:           .res 2
HFlipMask:                 .res 2
VFlipMask:                 .res 2
SpriteCount:               .res 2
NewObjectRequest:          .res 2
ObjectOffset:              .res 2

;Define Stuff

;Object Numbers
Player1 = 2
Bullet = 4

;Object Slot Registers
Identity = 0
XPosition = 2
YPosition = 4
XVelocity = 6
YVelocity = 8
HitboxWidth = 10
HitboxHeight = 12
Attributes = 14

;
; SNES RAM area, $0200-1fff
; 
; See lorom256k.cfg
;
; NOTE: Maybe consider using struct/endstruct for this stuff?  Might ask tepples or
; thefox how to use that for this kind of thing.  It would allow use of .sizeof()
; and not having to use *.
;
.segment "BSS"
SpriteBuf1:    .res $200   ; Sprite buffer for OAM DMA, 512 ($200) bytes
SpriteBuf1Size = *-SpriteBuf1
SpriteBuf2:    .res $20    ; Sprite buffer for OAM DMA, 32  ($20) bytes
SpriteBuf2Size = *-SpriteBuf2
SpriteBuf3:    .res $200   ; Sprite buffer for OAM DMA, 512  ($200) bytes
SpriteBuf3Size = *-SpriteBuf3
SpriteBufSize  = SpriteBuf1Size+SpriteBuf2Size
ObjectTable:    .res $1800

; Assemble-time assertion; SNES DMA transfer can only handle 65536 bytes.
.assert (SpriteBufSize < $10000), error, "SpriteBufs area too big for SNES DMA transfer"

;
; Program/code area
;
.segment "CODE"

.proc SpriteInit
  php	
  rep #$30

  lda #$01
  ldx #0
: sta SpriteBuf3+1,x
  inx
  inx
  inx
  inx
  cpx #$0200
  bne :-

  plp
  rts
.endproc


.proc Main
  ;
  ; Initialize SNES, which includes setting up CPU registers properly,
  ; zeroing out RAM contents, setting up SNES registers per official
  ; Nintendo documentation, and many other things.
  ;
  InitializeSNES

  ; Make sure registers are proper size before calling these macros!
  ; We could add this to each macro, but that's up to you.
  rep #$30   ; A=16, X/Y=16
  sep #$20   ; A=8
  
  ; Load palette to make our pictures look correct
  LoadPalette BGPalette,     0, 16

  ; Load Tile and Character data to VRAM
  LoadBlockToVRAM Player1Tiles,		$0000, $0040, $0000
  LoadBlockToVRAM Player1Tiles,		$0100, $0040, $0040
  LoadBlockToVRAM Player1Tiles,		$0020, $0040, $0080
  LoadBlockToVRAM Player1Tiles,		$0120, $0040, $00C0
  LoadBlockToVRAM Player1Tiles,		$0040, $0040, $0100
  LoadBlockToVRAM Player1Tiles,		$0140, $0020, $0140
  LoadBlockToVRAM Player1Tiles,		$0060, $0040, $0160
  LoadBlockToVRAM Player1Tiles,		$0160, $0040, $01A0
  LoadBlockToVRAM Player1Tiles,		$0080, $0020, $01E0
  LoadBlockToVRAM Player1Tiles,		$0180, $0020, $0200
  LoadBlockToVRAM Bullet1Tiles,		$00A0, $0040, $0000
  LoadBlockToVRAM BackgroundPics,	$2000, $3620, $0000	; 384 tiles * (8bit color)= 0x6000 bytes
  LoadBlockToVRAM BackgroundMap,	$7000, $1000, $0000	; 64x64 tiles = 4096 words = 8192 bytes

  jsr SpriteInit
  jsr JoyInit      ; setup joypads

  ; setup our walking sprite

  rep #$30                  ; A=16, X/Y=16
  lda #Player1
  sta NewObjectRequest
  jsr start_object_spawner

  ;
  ; Note from koitsu -- I'm not so sure this is right.  Go look at what the
  ; format of the OAM table is (official SNES manual, "OBJECT DATA", page A-3).
  ; To me this looks like it sets the Y position MSB (bit 9 of Y position) of
  ; sprites #2 and #3.  Am I wrong?
  ;

  ;
  ; Note from koitsu -- you realise that your sprites aren't actually set up
  ; quite yet?  SpriteInit just initialises SpriteBuf1 and SpriteBuf2
  ;

  jsr SetupVideo   ; setup the video modes and such, then turn on the screen

  ; Enable NMI (when interrupts get re-enabled) and joypad reading
  lda #$81
  sta $4200

  cli              ; Restore interrupts -- we're all ready to go!

InfiniteLoop:
  wai                         ; Wait for interrupts to finish (NMI/VBlank)
  jsr start_object_identifier
  jsr start_process_oam
  jmp InfiniteLoop            ; Do this forever

.endproc

;=========================================================================================
; VBlank
;=========================================================================================

.proc VBlank
  pha
  phx
  phy
  phb
  php

  rep #$30   ; A=16, X/Y=16
  sep #$20   ; A=8

  ; *********transfer sprite data

  stz $2102                    ; set OAM address to 0
  stz $2103                    ; ...and no priority rotation

  stz $4300                    ; CPU -> PPU, auto-increment, write once
  lda #$04
  sta $4301                    ; DMA will write to register $2104 (OAM data write)
  ldy #.LOWORD(SpriteBuf1)
  sty $4302                    ; source offset of SpriteBuf1
  lda #.BANKBYTE(SpriteBuf1)
  sta $4304                    ; bank address of SpriteBuf1
  ldy #SpriteBufSize
  sty $4305                    ; transfer SpriteBufSize bytes (pre-calculated number)
  lda #$01
  sta $420b                    ; start DMA transfer

  ; update the map co-ordinates
  lda MapX
  sta $210d
  lda MapX+1
  sta $210d

  lda MapY
  sta $210e
  lda MapY+1
  sta $210e

  jsr GetInput    ; update joypad data

  lda $4210       ; clear NMI Flag

  rep #$10        ; A=16
  inc FrameNum    ; increment 16-bit frame counter (if ever used/needed)

  plp
  plb
  ply
  plx
  pla
  rti
.endproc

;=========================================================================================
; SetupVideo
;=========================================================================================

.proc SetupVideo
  php
  rep #$30   ; A=16, X/Y=16
  sep #$20   ; A=8

  stz $211b
  lda #$01
  sta $211b
  stz $211c
  stz $211c
  stz $211d
  stz $211d
  stz $211e
  sta $211e
  lda #$80
  sta $211f
  stz $211f
  sta $2120
  stz $2120

  sta $2121
  lda #%01100000		;Sprites 32x32 or 64x64, character data at $0000 (word address)
  sta $2101         

  lda #$01		;Set video mode 3, 8x8 tiles (256 color BG1, 16 color BG2)
  sta $2105         

  lda #$71		;Set BG1's Tile Map VRAM offset to $7800 (word address)
  sta $2107		;   and the Tile Map size to 64 tiles x 32 tiles

  lda #$02		;Set BG1's Character VRAM offset to $4000 (word address)
  sta $210b

  lda #$11		;Turn on BG1 and BG2 and Sprites
  sta $212c

  lda #$ff		;Scroll BG2 down 1 pixel
  sta $2110
  sta $2110

  lda #$0f		;Turn on screen, full brightness
  sta $2100

  plp
  rts
.endproc

;=========================================================================================
; Segment for our binary data
;=========================================================================================
.segment "RODATA"
;=========================================================================================

MetaspriteTableSize:
  .word $0005   ; Number of metasprite table entries below

MetaspriteTable:
        ;XPos  YPos  Size  Palette/Character
  .word $0000,$0000,$0000,$0000
  .word $0000,$0010,$0000,$0002
  .word $0010,$000D,$0000,$0004
  .word -$0002,$0020,$0000,$0006
  .word $000E,$0020,$0000,$0008


Bullet1MetaspriteTableSize:
  .word $0001   ; Number of metasprite table entries below

Bullet1MetaspriteTable:
        ;XPos  YPos  Size  Palette/Character
  .word $0000,$0000,$0000,$000A

;=========================================================================================

; Map data
BackgroundMap:
  .incbin "GamePictures\hovertransport.map"

;=========================================================================================
; Color data
;=========================================================================================

BGPalette:
  .incbin "GamePictures\hovertransport.clr"

Player1Palette:
  .incbin "GamePictures\player1.clr"

BulletPalette:
  .incbin "GamePictures\bullet.clr"

;=========================================================================================
; Tile data
;=========================================================================================

BackgroundPics:
  .incbin "GamePictures\hovertransport.pic"

Player1Tiles:
  .incbin "GamePictures\player1.pic"

Bullet1Tiles:
  .incbin "GamePictures\bullet1.pic"
