.setcpu "65816"
.smart

; See MetaspriteTest.asm
.import VBlank, Main

MAPPER_LOROM   = $20
MAPPER_HIROM   = $21
ROMSPEED_200NS = $00
ROMSPEED_120NS = $10

; ROM and backup RAM sizes are expressed as log2(size in bytes) - 10
MEMSIZE_NONE   = $00
MEMSIZE_2KB    = $01
MEMSIZE_4KB    = $02
MEMSIZE_8KB    = $03
MEMSIZE_16KB   = $04
MEMSIZE_32KB   = $05
MEMSIZE_64KB   = $06
MEMSIZE_128KB  = $07
MEMSIZE_256KB  = $08
MEMSIZE_512KB  = $09
MEMSIZE_1MB    = $0A
MEMSIZE_2MB    = $0B
MEMSIZE_4MB    = $0C

REGION_JAPAN   = $00
REGION_AMERICA = $01
REGION_PAL     = $02

.segment "SNESHEADER"
romname:
  .byte "SNES - Walker Example"
  .assert * - romname <= 21, error, "ROM name too long"
  .if * - romname < 21
    .res romname + 21 - *, $20  ; space padding
  .endif
  .byte MAPPER_LOROM|ROMSPEED_200NS
  .byte $00              ; 00: no extra RAM; 02: RAM with battery
  .byte MEMSIZE_256KB    ; ROM size (08-0C typical)
  .byte MEMSIZE_NONE     ; backup RAM size (01,03,05 typical; Dezaemon has 07)
  .byte REGION_AMERICA   ; region code
  .byte $33              ; publisher id, or $33 for "see 16 bytes before header"
  .byte $00              ; ROM revision number
  .word $0000            ; Checksum of all bytes will be poked here after linking
  .word $0000            ; $FFFF minus above sum will also be poked here
  .res 4                 ; Unused vector space
  ; Native 65816 vectors
  ; NOTE: Reset vector set to $FFFF because it doesn't serve a purpose here;
  ; the 65816 starts up in emulation mode (even on soft reset).
  .addr EmptyHandler     ; 65816 COP vector
  .addr EmptyHandler     ; 65816 BRK vector
  .addr EmptyHandler     ; 65816 ABORT vector
  .addr VBlank           ; 65816 NMI vector
  .addr $0000            ; 65816 RESET vector -- see above
  .addr EmptyHandler     ; 65816 IRQ vector
  .res 4                 ; Unused vector space
  ; 6502/65c02 vectors
  ; NOTE: BRK vector set to $FFFF because 6502/65c02 doesn't have an actual
  ; BRK vector, it only has an IRQ vector.
  .addr EmptyHandler     ; 6502/65c02 COP vector
  .addr $0000            ; 6502/65c02 BRK vector -- see above
  .addr EmptyHandler     ; 6502/65c02 ABORT vector
  .addr EmptyHandler     ; 6502/65c02 NMI vector
  .addr Main             ; 6502/65c02 RESET vector
  .addr EmptyHandler     ; 6502/65c02 IRQ vector

;
; These are vectors which are essentially unused on the SNES, i.e.
; defined to meet ca65 design requirements.
;
; The only ones which could be used are cop_handler, brk_handler,
; abort_handler, and irq_handler, but at present those do not serve
; a purpose for this game.
;
.segment "CODE"

.proc EmptyHandler
	rti
.endproc

.proc EmptyVBlank
	rep #30
	pha
	php

	sep #$20
	lda $4210		;clear NMI Flag

	plp
	pla
	rti
.endproc
