.setcpu "65816"
.smart

; See Game.asm
.importzp Empty
.importzp MetaspriteTableOffset
.importzp MetaspritePalette
.importzp MetaspriteCharacterOffset
.importzp MetaspriteCount
.importzp HFlipMask
.importzp VFlipMask
.importzp SpriteCount
.importzp ObjectOffset
.import SpriteBuf1
.import SpriteBuf3
.import MetaspriteTable
.import ObjectTable

Identity = 0
XPosition = 2
YPosition = 4
XVelocity = 6
YVelocity = 8
HitboxWidth = 10
HitboxHeight = 12
Attributes = 14

; Used by Game.asm
.export start_metasprite

.segment "CODE"

;
; start_metasprite
;
; Upon entry:
;   X register = offset into MetaspriteTable to start at
;   Y register = offset to start writing at in SpriteBuf1
; 
.proc start_metasprite
  phx
  phy
  rep #$30    ; A=16, X/Y=16
  ldy a:SpriteCount
  ldx a:MetaspriteTableOffset

  stz a:HFlipMask
  lda Attributes
  bit #$4000
  beq vertical_flip_check
  lda #$FFFF
  sta a:HFlipMask

vertical_flip_check:
  stz a:VFlipMask
  lda Attributes
  bit #$8000
  beq metasprite_loop
  lda #$FFFF
  sta a:VFlipMask

metasprite_loop:
  cpy #$0200		    ; sees if all 128 sprites are used up
  beq done
  and #$00FF
  sta a:SpriteBuf1+1,y        ; Store sprite Y position in SpriteBuf1+1,y
  lda a:MetaspriteCount       ; If MetaspriteCount is zero, then we're done.  Otherwise we have
  beq done                  ; metasprites to iterate over and populate for DMA (see VBlank)

  lda a:Empty,x               ; 1st byte = sprite X position (value 0-255)
  eor a:HFlipMask
  clc
  adc XPosition
  cmp #256
  bcc sprite_x_not_out_of_bounds
  cmp #65504
  bcs sprite_x_not_out_of_bounds
  bra sprite_out_of_bounds

sprite_x_not_out_of_bounds:
  and #$01FF
  sta a:SpriteBuf3,y          ; Store sprite X position SpriteBuf1+y
  and #$00FF
  sta a:SpriteBuf1,y          ; Store sprite X position SpriteBuf1+y
  lda a:Empty+2,x             ; 1st byte = sprite X position (value 0-255)
  eor a:VFlipMask
  clc
  adc YPosition
  cmp #224
  bcc sprite_y_not_out_of_bounds
  cmp #65504
  bcs sprite_y_not_out_of_bounds
  bra sprite_out_of_bounds

sprite_y_not_out_of_bounds:
  sta a:SpriteBuf1+1,y
  lda a:Empty+4,x
  sta a:SpriteBuf3+2,y        ; Sprite size
  lda a:Empty+6,x
  eor Attributes
  sta a:SpriteBuf1+2,y        ; Palette/Character word
  txa
  clc
  adc #$0008
  tax
  iny
  iny
  iny
  iny
  dec a:MetaspriteCount       ; Decrement MetaspriteCount by 1
  bra metasprite_loop       ; Back to the loop...

sprite_out_of_bounds:
  txa
  clc
  adc #$0008
  tax
  dec a:MetaspriteCount       ; Decrement MetaspriteCount by 1
  bra metasprite_loop       ; Back to the loop...

done:
  sty a:SpriteCount	    ; Says how many sprites have been made
  ply
  plx
  rts

.endproc