.setcpu "65816"
.smart
.feature force_range

; See Metasprite.asm
.import start_metasprite

; See PaletteChecker.asm
.import start_palette_checker

; Used by Game.asm
.export start_object_spawner
.export start_object_identifier

; See Game.asm
.importzp Empty
.importzp Joy1Data
.importzp MapX
.importzp MapY
.importzp MetaspriteTableOffset
.importzp MetaspriteCount
.importzp NewObjectRequest
.importzp ObjectOffset
.import ObjectTable
.import MetaspriteTable
.import MetaspriteTableSize
.import Bullet1MetaspriteTable
.import Bullet1MetaspriteTableSize
.import TotalPalettesTable

;Object Numbers
Player1 = 2
Bullet = 4

;Object Slot Registers
Identity = 0
XPosition = 2
YPosition = 4
XVelocity = 6
YVelocity = 8
HitboxWidth = 10
HitboxHeight = 12
Attributes = 14

.segment "CODE"

;====================================================================================
; start_object_spawner
;====================================================================================

.proc start_object_spawner
  rep #$30    ; A=16, X/Y=16
  ldx #$0000

object_spawner_loop:
  lda a:ObjectTable,x
  beq object_spawner_loop_done
  txa
  clc
  adc #$0030
  tax
  cpx #6144
  bne object_spawner_loop

object_spawner_loop_done:
  lda a:NewObjectRequest
  sta a:ObjectTable,x
  rts

.endproc

;====================================================================================
; start_object_identifier
;====================================================================================

.proc start_object_identifier
  phd
  rep #$30    ; A=16, X/Y=16
  lda #ObjectTable
  sta a:ObjectOffset
  tcd

object_identifier_loop:
  lda $0000				;load the object identification byte of the object we're currently on
  beq next_object
  tax					;put the result in x so we can offset the object identification table to see what the object is
  jsr (ObjectIdentificationTable-2,x)	;jump to the code that corresponds with the object
  rep #$30    ; A=16, X/Y=16
next_object:
  lda a:ObjectOffset			;says how many objects have been identified
  clc
  adc #$0030				;add 48 to look at the next object
  sta a:ObjectOffset			;store the result for the next time we go through the loop
  tcd					;transfer the accumulator to y for "cpy #8192" and "lda ObjectTable,y"
  cmp #6144+ObjectTable			;sees if all 128 objects have been identified (each object is 128 bytes)
  bne object_identifier_loop		;if so, quit searching
  pld
  rts

.endproc

;====================================================================================
; ObjectIdentificationTable
;====================================================================================

ObjectIdentificationTable:
  .word player1,bullet

;====================================================================================
; Objects
;====================================================================================
; Player 1
;====================================================================================

.proc player1
  rep #$30                  ; A=16, X/Y=16

;decrements timer so you can shoot again

  lda $10
  beq y_button
  dec
  sta $10

y_button:

  lda a:Joy1Data
  and #$4000
  beq up

  lda $10
  bne up

  lda ObjectTable+Attributes
  and #%0111111111111111
  bne spawnbulletleft

  lda #$0004
  sta $10

  lda #Bullet
  sta a:NewObjectRequest
  jsr start_object_spawner
  lda XPosition
  clc
  adc #31
  sta a:ObjectTable+XPosition,x
  lda YPosition
  clc
  adc #14
  sta a:ObjectTable+YPosition,x
  lda Attributes
  and #$4000
  sta a:ObjectTable+Attributes,x
  lda #$00
  sta a:ObjectTable+XVelocity,x
  bra up

spawnbulletleft:

  lda #$0004
  sta ObjectTable+16

  lda #Bullet
  sta a:NewObjectRequest
  jsr start_object_spawner
  lda XPosition
  sec
  sbc #31
  sta a:ObjectTable+XPosition,x
  lda YPosition
  clc
  adc #14
  sta a:ObjectTable+YPosition,x
  lda Attributes
  and #$4000
  sta a:ObjectTable+Attributes,x
  lda #-$0B
  sta a:ObjectTable+XVelocity,x
  

up:
  lda a:Joy1Data
  and #$0800
  beq down

  lda YPosition
  sec
  sbc #$0002
  sta YPosition
  lda Attributes
  and #%0111111111111111
  sta Attributes

down:
  lda a:Joy1Data
  and #$0400
  beq left

  lda YPosition
  clc
  adc #$0002
  sta YPosition
  lda Attributes
  ora #$8000
  sta Attributes

left:
  lda a:Joy1Data
  and #$0200
  beq right

  dec a:MapX
  lda XPosition
  sec
  sbc #$0002
  sta XPosition
  lda Attributes
  ora #$4000
  sta Attributes

right:
  lda a:Joy1Data
  and #$0100
  beq done

  inc a:MapX
  lda XPosition
  clc
  adc #$0002
  sta XPosition
  lda Attributes
  and #%1011111111111111
  sta Attributes

done:

  lda MetaspriteTableSize   ; Load number of Metasprites
  sta a:MetaspriteCount       ; ...and store it in MetaspriteCount

  lda #MetaspriteTable      ; Offset into MetaspriteTable
  sta a:MetaspriteTableOffset

  jsr start_metasprite      ; jump to start_metasprite to build metasprites
  rts

.endproc

;====================================================================================
;Bullet
;====================================================================================

.proc bullet
  rep #$30                         ; A=16, X/Y=16

  lda XPosition
  cmp #256
  bcs terminate_bullet
  cmp #65520
  bcs terminate_bullet
  clc
  adc XVelocity
  sta XPosition

  lda YPosition
  cmp #224
  bcs terminate_bullet
  cmp #65520
  bcs terminate_bullet
  clc
  adc YVelocity
  sta YPosition

  lda Bullet1MetaspriteTableSize   ; Load number of Metasprites
  sta a:MetaspriteCount              ; ...and store it in MetaspriteCount

  ldx #Bullet1MetaspriteTable      ; Offset into MetaspriteTable
  stx a:MetaspriteTableOffset

  jsr start_metasprite      ; jump to start_metasprite to build metasprites
  rts

terminate_bullet:
  lda #$0000
  sta $0000
  sta Attributes
  rts

.endproc
